package com.csliubo.flutter.plugin.youme;

import android.util.Log;

import com.google.gson.Gson;
import com.youme.imsdk.ContactsSessionInfo;
import com.youme.imsdk.YIMClient;
import com.youme.imsdk.YIMConstInfo;
import com.youme.imsdk.YIMExtraUserInfo;
import com.youme.imsdk.YIMMessage;
import com.youme.imsdk.YIMMessageBodyAudio;
import com.youme.imsdk.YIMMessageBodyText;
import com.youme.imsdk.YIMService;
import com.youme.imsdk.callback.YIMEventCallback;
import com.youme.imsdk.internal.SendMessage;
import com.youme.imsdk.internal.UserStatusInfo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * YoumePlugin
 */
public class YoumePlugin implements MethodCallHandler, EventChannel.StreamHandler {
    private static final String METHOD_INIT = "init";
    private static final String METHOD_LOGIN = "login";
    private static final String METHOD_LOGOUT = "logout";
    private static final String METHOD_SEND_TEXT_MSG = "send_txt_msg";
    private static String mUserId = "444455555";
    private final static String mPassword = "123456";
    private final static String mChatRoomId = "12345";
    private Registrar registrar;
    private YouMeIMCallback mCallback = new YouMeIMCallback();
    private YoumeContactListener contactListener = new YoumeContactListener();
    private EventChannel.EventSink eventSink;

    /**
     * Plugin registration.
     */

    public YoumePlugin(Registrar registrar) {
        this.registrar = registrar;
    }

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel =
                new MethodChannel(registrar.messenger(), "youme_method");
        final EventChannel eventChannel =
                new EventChannel(registrar.messenger(), "youme_event");
        final YoumePlugin youmePlugin =
                new YoumePlugin(registrar);
        channel.setMethodCallHandler(youmePlugin);
        eventChannel.setStreamHandler(youmePlugin);
    }

    private static void print(Object msg) {
        Log.i("youme", "" + msg);
    }

    @Override
    public void onMethodCall(MethodCall call, final Result result) {
        String methodName = call.method;
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else if (methodName.equals(METHOD_INIT)) {
            String appKey = call.argument("appKey");
            String appSecret = call.argument("appSecret");
            Log.i("youme", "call init");
            YIMClient.getInstance().init(registrar.context(), appKey, appSecret, 0);
            YIMClient.getInstance().registerReconnectCallback(mCallback);
            YIMClient.getInstance().registerKickOffCallback(mCallback);
            YIMClient.getInstance().registerMsgEventCallback(mCallback);

//            YIMClient.getInstance().registerMessageListener();
            result.success("init success");
        } else if (methodName.equals(METHOD_LOGIN)) {
            String userId = call.argument("userId");
            String pwd = call.argument("pwd");
            Log.i("youme", "call login : " + userId + ", " + pwd);

            YIMClient.getInstance().login(userId, pwd, "", new YIMEventCallback.ResultCallback<String>() {
                @Override
                public void onSuccess(String userId) {
                    print("用户: " + userId + " 登录成功");
//                    YIMService.getInstance().registerContactListener(contactListener);
//                    print("contact listener return " + YIMService.getInstance().getHistoryContact());
                    YIMClient.getInstance().getHistoryContact(new YIMEventCallback.ResultCallback<ArrayList<ContactsSessionInfo>>() {

                        @Override
                        public void onSuccess(ArrayList<ContactsSessionInfo> contactsSessionInfos) {
                            print("onSuccess");
                            for (ContactsSessionInfo info : contactsSessionInfos) {
                                print(info);
//                                info.
//                                YIMClient.getInstance().queryHistoryMessage(info.strContactID, YIMConstInfo.ChatType.PrivateChat);
                            }
                            print("onSuccess==");
                        }

                        @Override
                        public void onFailed(int i, ArrayList<ContactsSessionInfo> contactsSessionInfos) {
                            print("onFailed");
                            for (ContactsSessionInfo info : contactsSessionInfos) {
                                print(info);
                            }
                            print("onFailed==");
                        }
                    });
                }

                @Override
                public void onFailed(int errorCode, String userId) {
                    print("用户: " + userId + " 登录失败:" + errorCode);
                }
            });

        } else if (methodName.equals(METHOD_LOGOUT)) {
            YIMClient.getInstance().logout(new YIMEventCallback.OperationCallback() {
                @Override
                public void onSuccess() {
                    print("用户: " + mUserId + " 已经退出登录");
                }

                @Override
                public void onFailed(int i) {
                    print("用户: " + mUserId + " 已经退出登录，但是可能本来就没有登录");
                }
            });

        } else if (methodName.equals(METHOD_SEND_TEXT_MSG)) {
            String receiver = call.argument("receiver");
            final String content = call.argument("content");

            if ("".equalsIgnoreCase(receiver) || "".equalsIgnoreCase(content)) {
                return;
            }
            YIMClient.getInstance().sendTextMessage(receiver, YIMConstInfo.ChatType.PrivateChat, content, new YIMEventCallback.ResultCallback<SendMessage>() {
                @Override
                public void onSuccess(SendMessage sendMessage) {
                    print("发送消息成功: " + content);
                    result.success("success");
                }

                @Override
                public void onFailed(int i, SendMessage sendMessage) {
                    print("发送消息失败: " + content);
//                    result.error(String.valueOf(sendMessage.getReasonType()));
                }
            });
//            YIMClient.getInstance().on
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onListen(Object o, EventChannel.EventSink eventSink) {
        this.eventSink = eventSink;
    }

    @Override
    public void onCancel(Object o) {

    }

    private class YoumeContactListener implements YIMService.ContactListener {

        @Override
        public void onGetContact(ArrayList<ContactsSessionInfo> arrayList) {
            print("onGetContact");
            for (ContactsSessionInfo info : arrayList) {
                print(info);
            }
            print("onGetContact==");
        }

        @Override
        public void onGetUserInfo(Integer integer, YIMExtraUserInfo yimExtraUserInfo) {
            print("onGetUserInfo");
            print(integer);
            print(yimExtraUserInfo);
            print("onGetUserInfo==");
        }

        @Override
        public void onQueryUserStatus(Integer integer, UserStatusInfo userStatusInfo) {
            print("onQueryUserStatus");
            print(integer);
            print(userStatusInfo);
            print("onQueryUserStatus==");
        }
    }

    private class YouMeIMCallback implements YIMEventCallback.ReconnectCallback, YIMEventCallback.MessageEventCallback, YIMEventCallback.KickOffCallback {


        /**
         * 接收到用户发来的消息
         *
         * @param message 消息内容结构体
         */
        public void onRecvMessage(YIMMessage message) {
            if (null == message)
                return;
            int msgType = message.getMessageType();
            if (YIMConstInfo.MessageBodyType.TXT == msgType) {
                Log.i("youme", "接收到一条文本消息： " + ((YIMMessageBodyText) message.getMessageBody()).getMessageContent());
                Map<String, Object> ret = new HashMap<String, Object>();
                ret.put("senderId", message.getSenderID());
                ret.put("isPrivate", true);
                ret.put("content", ((YIMMessageBodyText) message.getMessageBody()).getMessageContent());

                eventSink.success(new Gson().toJson(ret));
//                eventSink.success(message);
            } else if (YIMConstInfo.MessageBodyType.Voice == msgType) {
                YIMMessageBodyAudio audioMessage = (YIMMessageBodyAudio) message.getMessageBody();

                int audioDuration = audioMessage.getAudioTime();//录音时长，秒
                String param = audioMessage.getParam();//自定义内容
                String recognizeText = audioMessage.getText();//语音转文字的识别结果
                eventSink.success(message);
//                showRecv("接收到一条语音消息请下载");
//                mAudioDownloaded = false;
//                //回调是在子线程，渲染需要回到主线程
//                mDownloadButton.post(new Runnable() {
//                    @Override
//                    public void run() {
//                        mDownloadButton.setEnabled(true);
//                        mDownloadButton.setText(R.string.download);
//                    }
//                });
//
//                mRecvAudioMsgId = message.getMessageID();//下载用的消息id
            }
        }

        /**
         * 语音的识别文本回调
         *
         * @param errorcode 错误码
         * @param requestID 消息ID
         * @param text      返回的语音识别文本
         */
        @Override
        public void onGetRecognizeSpeechText(int errorcode, long requestID, String text) {

        }

        /*
         * 功能：录音音量变化回调, 频率:1s 约8次
         * @param volume：音量值(0到1)
         */
        @Override
        public void onRecordVolume(float volume) {

        }

        /**
         * 开始重连通知
         */
        @Override
        public void onStartReconnect() {

        }

        /**
         * 重连结果通知
         *
         * @param result 0-重连成功，1-重连失败，再次重连，2-重连失败
         */
        @Override
        public void onRecvReconnectResult(int result) {

        }

        /**
         * 被踢下线通知
         */
        @Override
        public void onKickOff() {

        }

        /**
         * 功能：新消息通知（默认自动接收消息，只有调用setReceiveMessageSwitch设置为不自动接收消息，才会收到该回调），有新消息的时候会通知该回调，频道消息会通知消息来自哪个频道ID
         *
         * @param chatType：聊天类型
         * @param targetID：频道ID
         */
        @Override
        public void onRecvNewMessage(int chatType, String targetID) {

        }
    }
}
